
#define IDLE_STATE 0
#define WANDER_STATE 1
#define PURSUIT_STATE 2
#define CAUGHT_STATE 3


Actor/Fish

	mass = 10

	damping = 0.075

	step_size = 1
	density = 0

	var
		saleValue
		points

		//the attraction threshholds are minimums to grab the attention of the fish
		shinyAttraction
		smellyAttraction
		vibrationAttraction


		fightProb = 20

		fightAccel = 300
		fightDelay = 1
		fightTimer

		fightEnergy = 50

		pushAccel = 100
		pushDelay = 0.5
		pushTimer

		maxSpeed = 80

		state = WANDER_STATE
		isActive = 1

		Actor/Lure/target

		stateTimer

		searchFront = 1
		searchUp = 1
		searchDown = 1
		searchBack = 1

		moveDir

		mouthOffsetForward
		mouthOffsetY

		fishLength = 3


		//defining a horizontal range that he likes
		maxx
		minx

		xRange = 6

		vector/wanderDir = new/vector(-1,0,0)

	updateStep()
		if(isActive)
			if(stateTimer < gameTime)
				StateMachine()

			if(!locate(/Water) in obounds(src))
				velocity.y -= 10

			.=..()

			dir = moveDir

	New()
		.=..()
		activeActors += src

		if(dir & EAST)
			maxx = min(world.maxx, x + xRange)
			minx = max(1,x - 1)
		else
			maxx = min(world.maxx, x + 1)
			minx = max(1,x - xRange)

	Bump(atom/A)
		.=..(A)

		if(istype(A,/Actor/Lure)) return

		velocity.x = 0
		velocity.y = 0

		wanderDir.x *= -1

	proc
		StateMachine()
			switch(state)
				if(IDLE_STATE) //idle
					IdleState()

				if(WANDER_STATE) //wander
					WanderState()

				if(PURSUIT_STATE) //pursuing hook
					PursuitState(target)

				if(CAUGHT_STATE) //caught
					CaughtState()



		IdleState()
			stateTimer = gameTime + rand(1,100) / 100

			if(prob(30)) state = WANDER_STATE
			SearchForHook()


		WanderState()

			var/r = prob(3)

			if(r)
				wanderDir.x *= -1

			if(x < minx) wanderDir.x = 1
			if(x > maxx) wanderDir.x = -1

			Push(wanderDir)

			if(prob(2)) state = IDLE_STATE
			SearchForHook()



		PursuitState(Actor/Lure/H)

			if(!H || H.caughtFish)
				target = null
				state = IDLE_STATE
				return

			var
				dx = (H.x - src.x) * world.icon_size + H.step_x - src.step_x
				dy = (H.y - src.y) * world.icon_size + H.step_y - src.step_y

			var/vector/V = new(dx, dy, 0)

			Push(V)

			if(H in obounds(src))
				H.caughtFish = 1
				state = CAUGHT_STATE


		CaughtState()

			if(!target)
				state = 1
				return

			moveDir = WEST

			loc = target.loc
			step_x = target.step_x + mouthOffsetForward
			step_y = target.step_y - mouthOffsetY

			if(istype(target,/Actor/Lure))
				target:fish = src


			if(!target)
				state = 1
				return

			if(!fightAccel) return
			if(gameTime < fightTimer) return
			if(!prob(fightProb)) return
			if(!fightEnergy) return

			fightTimer = gameTime + fightDelay
			fightEnergy --

			var/vector/fightDir = new(-1, 0.5 * pick(-1,0),0)

			target.velocity = target.velocity.add(fightDir.multiply(fightAccel))

			for(var/Water/W in obounds(src,4))
				W.velocity = src.velocity

			if(istype(target,/Actor/Lure))
				target:incrementFight()

		Push(vector/V)
			if(gameTime < pushTimer) return
			pushTimer = gameTime + pushDelay

			V = V.normalize()
			velocity = velocity.add(V.multiply(pushAccel))

			if(velocity.x < 0) moveDir = WEST
			if(velocity.x > 0) moveDir = EAST

			for(var/Water/W in obounds(src,4))
				W.velocity = src.velocity

		SearchForHook()
			var
				x1 = x * world.icon_size + step_x
				y1 = y * world.icon_size + step_y - searchDown
				x2 = x * world.icon_size + step_x + bound_width
				y2 = y * world.icon_size + step_y + bound_height + searchUp

			if(dir & WEST)
				x1 -= searchFront
				x2 += searchBack

			if(dir & EAST)
				x1 -= searchBack
				x2 += searchFront

			for(var/Actor/Lure/H in bounds(x1, y1, x2-x1, y2-y1, src.z))
				if(H && !H.caughtFish)
					target = H
					break

			if(target)
				if(HookAttracts(target))
					state = PURSUIT_STATE

		HookAttracts(Actor/Lure/H)

			var/luck = rand(-11,11)/10
			if(H.shiny + luck > src.shinyAttraction)
				return 1


			luck = rand(-11,11)/10
			if(H.smelly + luck > src.smellyAttraction)
				return 1

			luck = rand(-11,11)/10
			if(H.vibration + luck > src.vibrationAttraction)
				if(H.velocity.vectorLengthSquared() > 150*150)
					return 1

		score()
			player.score += points
			player.money += saleValue



	Crappie
		//Crappies are really cheap and shitty fish to catch, but easy to catch
		//They are attracted to any bait/lure, so long as it is slightly above themselves.
		//They swim horizontally back and forth

		saleValue = 10
		points = 10

		fightEnergy = 15

		icon = 'SmallFish.dmi'
		icon_state = "crappie"

		shinyAttraction = 0
		smellyAttraction = 6
		vibrationAttraction = 4

		bound_x = 2
		bound_y = 5
		bound_width = 10
		bound_height = 7

		mouthOffsetForward = 0
		mouthOffsetY = 2

		fightAccel = 200
		fightDelay = 1.5

		searchFront = 16
		searchUp = 16
		searchDown = -8
		searchBack = 1

		xRange = 3

		New()
			.=..()
			fishLength = rand(2,4)



	Sunfish
		icon = 'SmallFish.dmi'
		icon_state = "sunfish"
		name = "Copperscale"

		saleValue = 25
		points = 25

		bound_x = 1
		bound_y = 1
		bound_width = 30
		bound_height = 14

		searchFront = 48
		searchUp = 1
		searchDown = 1
		searchBack = 1

		mouthOffsetForward = 1
		mouthOffsetY = 7

		shinyAttraction = 5
		smellyAttraction = 2
		vibrationAttraction = 2


		fightAccel = 175
		fightDelay = 0.25

		fightEnergy = 200

		New()
			.=..()
			fishLength = rand(6,16)

	Spiky
		icon = 'SmallFish.dmi'
		icon_state = "spekky"

		name = "Spiky Northfin"

		saleValue = 40
		points = 40

		bound_x = 1
		bound_y = 1
		bound_width = 30
		bound_height = 14

		searchFront = 48
		searchUp = 1
		searchDown = 1
		searchBack = 1

		mouthOffsetForward = 3
		mouthOffsetY = 10

		shinyAttraction = 2
		smellyAttraction = 6
		vibrationAttraction = 5

		pushAccel = 125
		pushDelay = 1

		fightAccel = 1400
		fightDelay = 1.5

		fightEnergy = 200

		New()
			.=..()
			fishLength = rand(10,20)

	ThunderEel
		name = "Thunder Eel"
		icon = 'Eel.dmi'

		saleValue = 85
		points = 85

		bound_x = 1
		bound_y = 1
		bound_width = 62
		bound_height = 14

		searchFront = 8
		searchUp = 16
		searchDown = 32
		searchBack = 8

		mouthOffsetForward = 3
		mouthOffsetY = 2

		shinyAttraction = 7
		smellyAttraction = 3
		vibrationAttraction = 3

		pushAccel = 80
		pushDelay = 0.5

		fightAccel = 450
		fightDelay = 0.25

		fightEnergy = 200

		New()
			.=..()
			fishLength = rand(15,50)

		StateMachine()

			.=..()

			if(state == PURSUIT_STATE || state == CAUGHT_STATE)
				icon_state = "pursuit"
			else
				icon_state = ""

	Demontooth
		name = "Abyssal Deviltooth"
		icon = 'demontooth.dmi'

		saleValue = 500
		points = 500

		bound_x = 7
		bound_y = 16
		bound_width = 114
		bound_height = 32

		mouthOffsetForward = -8//15
		mouthOffsetY = 30

		shinyAttraction = 5
		smellyAttraction = 6
		vibrationAttraction = 6


		fightAccel = 700
		fightDelay = 0.25

		fightEnergy = 300

		searchFront = 96
		searchUp = 1
		searchDown = 1
		searchBack = 1

		pushAccel = 75
		pushDelay = 2

		New()
			.=..()
			fishLength = rand(32,96)

		StateMachine()
			pushAccel = 75
			pushDelay = 1

			.=..()

			if(state == PURSUIT_STATE)
				icon_state = "open"
			else
				icon_state = ""


		PursuitState(Actor/Lure/H)
			pushAccel = 400
			pushDelay = 0.25
			.=..(H)