
Water
	parent_type = /obj

	icon = 'Water.dmi'
	icon_state = "1"
	invisibility = 101

	density = 0
	var
		waterRadius = 18

		elasticity = 8
		damping = 0.0075//0.0001//0.2

		offset_x = 0
		offset_y = 0

		obj/left
		obj/right

		istate

		tri_init = 0

		maxSpeed = 80

		updateDelay = 0.04

		ax = 0
		ay = 16

		bx = 16
		by = 16

		cx = 0
		cy = 0



	dx = 16
	dy = 0

	alpha = 32

	proc/init()

		bound_width = waterRadius
		bound_height = waterRadius

		istate = "[rand(1,3)]"

		left = new(src.loc)
		right = new(src.loc)

		left.icon = 'Water.dmi'
		right.icon = 'Water.dmi'

		left.icon_state = "[istate] L"
		right.icon_state = "[istate] R"

		left.plane = WATER_PLANE
		right.plane = WATER_PLANE

		left.alpha = alpha
		right.alpha = alpha

		left.mouse_opacity = 0
		right.mouse_opacity = 0

		activeWater += src

		var/joinDirs = 0
		for(var/Water/W in obounds(src))
			var/d = get_dir(src.loc, W.loc)
			if(d == NORTH) joinDirs |= NORTH
			if(d == EAST) joinDirs |= EAST
			if(d == WEST) joinDirs |= WEST
			if(d == SOUTH) joinDirs |= SOUTH


		if((joinDirs & NORTH) == 0)
			addTop()


		offset_x = rand(-8,8)
		offset_y = rand(-8,8)

		velocity.x = rand(-16,16)
		velocity.y = rand(-16,16)


	Del()
		del left
		del right

		activeWater -= src

		.=..()

	updateStep()
		//okay so first apply the second order differential equation for damping on the water...
		//numerical integration with euler's because 24 hours
		updateDelay = 0.04

		if(world.tick_lag > 10) updateDelay = 0.7 + rand(0,10)/10
		if(world.tick_lag > 20) updateDelay = 0.13 + rand(0,10)/10
		if(world.tick_lag > 30) updateDelay = 0.16 + rand(0,10)/10

		updateTimer = gameTime + updateDelay

		var/vector/acceleration = new(
			-offset_x * elasticity * 0.5,
			-offset_y * elasticity * 0.5,
			0)

		velocity = velocity.add(acceleration)

		velocity = velocity.multiply(1-damping)

		for(var/Actor/A in bounds(src))
			var/attenuation = 1 - (pixelDistanceSquared(src,A) / waterRadius / waterRadius)
			if(attenuation <= 0) continue

			A.velocity = A.velocity.add(acceleration.multiply(attenuation / A.mass))

		var/neighbors[0]

		for(var/Water/A in obounds(src))
			neighbors += A

		for(var/Water/A in neighbors)
			var/d = get_dir(src,A)
			if(d != NORTH || d != EAST || d != SOUTH || d != WEST) continue

			var/viscocity = 0.25 / neighbors.len

			A.velocity = A.velocity.add(velocity.multiply(viscocity))
			var/vector/V = getWaterOffsetVector(A)

			var/forceMult = (waterRadius - V.vectorLength())
			if(abs(forceMult) > 4)
				V = V.normalize()
				acceleration = V.multiply(forceMult * viscocity * elasticity * 10000)

				A.velocity = A.velocity.add(acceleration)

		applyVelocity()

	applyVelocity()
		if(!velocity) return
		if(velocity.x == 0 && velocity.y == 0) return

		velocity.x = max(-maxSpeed, min(velocity.x, maxSpeed))
		velocity.y = max(-maxSpeed, min(velocity.y, maxSpeed))

		var/speedMult = 1

		if(updateDelay > 0)
			speedMult *= round(updateDelay / world.tick_lag * 10)
			speedMult = max(1, speedMult)

		offset_x += velocity.x * deltaTime * speedMult
		offset_y += velocity.y * deltaTime * speedMult

		offset_x = max(-48, min(offset_x, 48))
		offset_y = max(-48, min(offset_y, 48))

	proc
		getWaterOffsetVector(Water/B)
			var/vector/V = new(0,0,0)

			V.x = (B.x - x) * world.icon_size + (B.offset_x - offset_x)
			V.y = (B.y - y) * world.icon_size + (B.offset_y - offset_y)

			return V

		addTop()
			left.overlays.Cut()
			left.overlays += new/icon('Water.dmi', "top")

		updateTriangles() //oh boy is this math heavy
			var
				hasChange = 0

			if(offset_x != cx || offset_y != cy)
				hasChange = 1

				cx = offset_x
				cy = offset_y

			var
				vector
					xScale
					yScale

					diagonal
					perpendicular

					up = new(0,0,1)

				rotation
				shearAmount

				triangle_size = world.icon_size

				matrix/triTransform

			for(var/Water/A in obounds(src))
				var/d = get_dir(src.loc,A.loc)
				if(d == NORTH)
					if( ax != A.offset_x || ay != A.offset_y + 16 )
						hasChange = 1

						ax = A.offset_x
						ay = A.offset_y + 16

				if(d == EAST)
					if( ax != A.offset_x + 16 || ay != A.offset_y )
						hasChange = 1

						dx = A.offset_x + 16
						dy = A.offset_y

				if(d == NORTHEAST)
					if( ax != A.offset_x + 16 || ay != A.offset_y + 16 )
						hasChange = 1

						bx = A.offset_x + 16
						by = A.offset_y + 16

			if(!hasChange && tri_init)
				return

			tri_init = 1

			var/vector
				A = new(ax, ay, 0)
				B = new(bx, by, 0)
				C = new(cx, cy, 0)
				D = new(dx, dy, 0)


			//right triangle
			xScale = D.subtract(C)
			diagonal = B.subtract(D)

			perpendicular = up.cross(xScale)

			if(diagonal.dot(perpendicular) == 0)
				perpendicular = perpendicular.add(xScale.multiply(0.01))

			yScale = diagonal.projectOnto(perpendicular)

			var/xLength = xScale.vectorLength()
			var/yLength = yScale.vectorLength()

			yLength = max(0.0000001,yLength)

			var/vector/xNormalized = xScale.normalize()

			rotation = arctan2(xScale.x, xScale.y)
			shearAmount = diagonal.dot(xNormalized)/yLength

			triTransform = new()
			triTransform.Translate(0.5 * triangle_size, 0.5 * triangle_size)
			triTransform.Scale(xLength / triangle_size, yLength / triangle_size)
			triTransform.Shear(shearAmount,0)
			triTransform.Turn(-rotation)

			triTransform.Translate(-0.5 * triangle_size, -0.5 * triangle_size)
			triTransform.Translate(cx, cy)

			right.transform = triTransform


			//left triangle

			xScale = A.subtract(B)
			diagonal = C.subtract(A)

			perpendicular = up.cross(xScale)

			if(diagonal.dot(perpendicular) == 0)
				perpendicular = perpendicular.add(xScale.multiply(0.01))

			yScale = diagonal.projectOnto(perpendicular)

			xLength = xScale.vectorLength()
			yLength = yScale.vectorLength()

			yLength = max(0.0000001,yLength)

			xNormalized = xScale.normalize()

			rotation = arctan2(xScale.x, xScale.y)
			shearAmount = diagonal.dot(xNormalized)/yLength

			triTransform = new()
			triTransform.Translate(0.5 * triangle_size, 0.5 * triangle_size)
			triTransform.Scale(xLength / triangle_size, yLength / triangle_size)
			triTransform.Shear(shearAmount,0)
			triTransform.Turn(-rotation)

			triTransform.Translate(-0.5 * triangle_size, -0.5 * triangle_size)
			triTransform.Translate(cx, cy)

			var/vector/offset = B.subtract(C)
			triTransform.Translate(offset.x, offset.y)

			left.transform = triTransform


obj
	backdrop
		icon = 'mapbackground.png'
		layer = BACKGROUND_PLANE

		landscape
			icon = 'landscape.png'